#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include "upload.h"

char *myname;

/*
 *  Set the serial port speed
 */
void setspeed(int speed, char *device)
{
	struct termios *term;
	int file;
	
	term = (struct termios *)malloc(sizeof(struct termios));
	
	file = open(device, O_RDWR | O_NOCTTY | O_SYNC);
	if (file == -1)
	{
		printf("%s: open() failed\n", myname);
		exit(0);
	}
	
	tcflush(file, TCIOFLUSH);
	
	if (tcgetattr(file, term))
	{
		printf("%s: tcgetattr() error\n", myname);
		close(file);
		exit(0);
	}
	
	cfsetispeed(term, B0);
	cfsetospeed(term, speed);
	
	if (tcsetattr(file, TCSANOW, term))
	{
		printf("%s: tcsetattr() failed\n", myname);
		close(file);
		exit(0);
	}
	
	close(file);
}

/*
 *  Main program loop
 */
int main(int argc, char *argv[])
{
	unsigned char c, ret, prog[EE_SIZE];
	int count, error;
	unsigned int num, address;
	FILE *commport, *S19file;
	
	myname = strdup(argv[0]);
	
	/*
	 *  With no arguments, expect the S19 data on STDIN
	 */
	if (argc == 1)
		S19file = stdin;
	else
		if ((S19file = fopen(argv[1], "r")) == NULL)
		{
			printf("%s: Cannot open %s.", argv[0], argv[1]);
			exit(-1);
		}

	/*
	 *  Since the S19 file is not a memory image, may contain gaps, and
	 *  may not be ordered, create a memory image and zero it to the
	 *  erased memory state - 0xff
	 */

	for (count = 0; count < EE_SIZE; count++)
		prog[count] = 0xFF;

	/*
	 *  Parse the S19 file and fill in the memory image with the data
	 */

	c = fgetc(S19file);
	while (!feof(S19file))
	{
		if (c == 'S')
			if (fgetc(S19file) == '1')
			{
				fscanf(S19file, "%2x%4x", &num, &address);
				for(count = 0; count < (num - 3); count++)
				{
					fscanf(S19file, "%2x", &prog[address - EE_START]);
					address++;
				}
			}
		c = fgetc(S19file);
	}
	
	/*
	 *  Initialise the serial port and flush it's buffers
	 */

	setspeed(B1200, DEVICE);
	
	/*
	 *  Open the port for R/W
	 */

	if ((commport = fopen(DEVICE, "r+")) == NULL)
	{
		printf("%s: Cannot open %s\n", myname, DEVICE);
		exit(-1);
	}

	/*
	 *  Expect the HC11 to be in Special Bootstrap mode and have just
	 *  been reset, so send a 0xff to init it's serial port to 1200 baud
	 */
	
	fputc(0xff, commport);
	
	/*
	 *  Wait a bit, the HC11 needs to see a clean 0xff without other
	 *  bits around to confuse it
	 */

 	usleep(10000); 

	/*
	 *  Now we loop through the micro-loader image.  Each byte is echoed
	 *  back to us to we can do some error checking.
	 */
	
	error = 0;
	for(count = 0; count < 256; count++)
	{
		c = image[count];
		fputc(c, commport);
		ret = fgetc(commport);
		if (c != ret)
		{
			error = count;
			printf("\nSent: %x Got: %x Count: %d\n", c, ret, count);
		}
		else
#ifdef PERCENT
		printf("Uploading: %d\r", count);
#else
		printf("Uploading: %c\r", spin[count%4]);
#endif
	}
	if (error)
		printf("Hmmm, we got an error.  Lets try to ignore it and continue.....\n");
	else
		printf("\n");
	printf("EEPROM programmer upload complete.\n");

	/*
	 *  Wait a bit, so the micro-programmer can init the serial port at
	 *  9600 baud, otherwise a fast host can transmit the first character
	 *  of the program image at 9600 baud while the HC11 is still at 1200
	 */

 	usleep(10000);
	setspeed(B9600, DEVICE);

	/*
	 *  Loop through the program image array.  The micro-programmer is
	 *  written to work similarly to the bootstrap loader, echoing the
	 *  actual byte from the EEPROM after writing.  So, if the EEPROM
	 *  is not written correctly, we will detect it.
	 */	

	error = 0;
	for (count = 0; (count < EE_SIZE); count++)
	{
		fputc(prog[count], commport);
		c = fgetc(commport);
		if (c != prog[count])
		{
			printf("\nError at 0x%4x: Sent 0x%02x  Got 0x%02x\n", count + EE_START, prog[count], c);
			error = count;
		}
		else
#ifdef PERCENT
			printf("Uploading: %d\r", count);
#else
			printf("Uploading: %c\r", spin[count%4]);
#endif
	}

	if (error)
		printf("User code echo-check failed.  Last byte to fail was %d\n", error);
	else
		printf("\n");
	printf("User code upload complete.\n");

	fclose(commport);
	
	exit(0);
}